'use strict';

/**
 * @file: index
 * @author: Cuttle Cong
 * @date: 2017/11/29
 * @description:
 */
var getSpecImport = require('./utils/getSpecImport');
var matchRule = require('./utils/matchRule');

var _require = require('../package.json'),
    symbol = _require.name;

function warn() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  console.warn.apply(console, [symbol + ' Warn...\n'].concat(args));
}

module.exports = function (babel) {
  return {
    pre: function pre(path) {
      this.runtimeData = {};
    },

    visitor: {
      ImportDeclaration: function ImportDeclaration(path, data) {
        var _this = this;

        data.opts.ignores = [];
        var locals = getSpecImport(path, { withPath: true, ignore: data.opts.ignore });
        if (locals) {
          locals.forEach(function (pathData, index, all) {
            var name = pathData.name;
            // already existed

            if (_this.runtimeData[name]) {
              warn('the declare of ', '`' + name + '`', 'is already existed');
              return;
            }

            _this.runtimeData[name] = {
              parent: path,
              children: all,
              data: pathData
            };
          });

          path.skip();
        }
      },
      Identifier: function Identifier(path, data) {
        var parentPath = path.parentPath;
        var name = path.node.name;
        // const ID = 'value';

        if (parentPath.isVariableDeclarator() && parentPath.get('id') === path) {}
        // { Tabs: 'value' }
        else if (parentPath.isLabeledStatement() && parentPath.get('label') === path) {}
          // ref.ID
          else if (parentPath.isMemberExpression() && parentPath.get('property') === path) {}
            // class A { ID() {} }
            else if ((parentPath.isClassProperty() || parentPath.isClassMethod()) && parentPath.get('key') === path) {} else {
                // used
                if (this.runtimeData[name]) {
                  delete this.runtimeData[name];
                }
              }
      }
    },
    post: function post(path) {
      var _this2 = this;

      /*
       {
         parent: path,
         children: [ { path, name } ],
         data: { path, name }
       }
       */
      var allNames = Object.keys(this.runtimeData);
      // warn(allNames)
      allNames.forEach(function (name) {
        var _runtimeData$name = _this2.runtimeData[name],
            children = _runtimeData$name.children,
            data = _runtimeData$name.data,
            parent = _runtimeData$name.parent;

        var childNames = children.map(function (x) {
          return x.name;
        });
        // every imported identifier is unused
        if (childNames.every(function (cName) {
          return allNames.includes(cName);
        })) {
          !parent.__removed && parent.remove();
          parent.__removed = true;
        } else {
          !data.path.__removed && data.path.remove();
          data.path.__removed = true;
        }
      });

      delete this.runtimeData;
    }
  };
};